@@ -7,7 +7,32 @@
 
 #include "drivemount.h"
 
-void property_load(gchar *path, DriveData *dd)
+static gchar *remove_level_from_path(const gchar *path)
+{
+	gchar *new_path;
+	const gchar *ptr;
+	gint p;
+
+	if (!path) return NULL;
+
+	p = strlen(path) - 1;
+	if (p < 0) return NULL;
+
+	ptr = path;
+	while(ptr[p] != '/' && p > 0) p--;
+
+	if (p == 0 && ptr[p] == '/') p++;
+	new_path = g_strndup(path, (guint)p);
+	return new_path;
+}
+
+static void sync_mount_base(DriveData *dd)
+{
+	g_free(dd->mount_base);
+	dd->mount_base = remove_level_from_path(dd->mount_point);
+}
+
+void property_load(const gchar *path, DriveData *dd)
 {
         gnome_config_push_prefix (path);
 
@@ -15,8 +40,11 @@ void property_load(gchar *path, DriveData *dd)
 	dd->device_pixmap = gnome_config_get_int("mount/pixmap=0");
 	dd->scale_applet = gnome_config_get_int("mount/scale=0");
 
+	dd->auto_eject = gnome_config_get_int("mount/auto_eject=0");
+
 	g_free(dd->mount_point);
 	dd->mount_point = gnome_config_get_string("mount/mountpoint=/mnt/floppy");
+	sync_mount_base(dd);
 	dd->autofs_friendly = gnome_config_get_int("mount/autofs_friendly=0");
 
 	g_free(dd->custom_icon_in);
@@ -38,7 +66,7 @@ void property_load(gchar *path, DriveData *dd)
 	gnome_config_pop_prefix ();
 }
 
-void property_save(gchar *path, DriveData *dd)
+void property_save(const gchar *path, DriveData *dd)
 {
         gnome_config_push_prefix(path);
 
@@ -46,6 +74,8 @@ void property_save(gchar *path, DriveData *dd)
         gnome_config_set_int("mount/pixmap", dd->device_pixmap);
         gnome_config_set_int("mount/scale", dd->scale_applet);
 
+	gnome_config_set_int("mount/auto_eject", dd->auto_eject);
+
         gnome_config_set_string("mount/mountpoint", dd->mount_point);
         gnome_config_set_int("mount/autofs_friendly", dd->autofs_friendly);
 
@@ -77,6 +107,13 @@ static void scale_applet_cb(GtkWidget *w, gpointer data)
 	gnome_property_box_changed(GNOME_PROPERTY_BOX(dd->propwindow));
 }
 
+static void auto_eject_applet_cb(GtkWidget *w, gpointer data)
+{
+	DriveData *dd = data;
+	dd->prop_auto_eject = GTK_TOGGLE_BUTTON (w)->active;
+	gnome_property_box_changed(GNOME_PROPERTY_BOX(dd->propwindow));
+}
+
 static void pixmap_floppy_cb(GtkWidget *widget, gpointer data)
 {
 	DriveData *dd = data;
@@ -137,7 +174,7 @@ static void pixmap_custom_cb(GtkWidget *widget, gpointer data)
         widget = NULL;
 }
 
-static void update_delay_cb( GtkWidget *widget, gpointer data)
+static void update_delay_cb(GtkWidget *widget, gpointer data)
 {
 	DriveData *dd = data;
         dd->prop_interval = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dd->prop_spin));
@@ -146,15 +183,20 @@ static void update_delay_cb( GtkWidget *widget, gpointer data)
         widget = NULL;
 }
 
-static void property_apply_cb( GtkWidget *widget, void *data, DriveData *dd)
+static void property_apply_cb(GtkWidget *propertybox, gint page_num, DriveData *dd)
 {
 	gchar *new_file;
 
+	if(page_num != -1) return;	/* only do this once, on global signal */
+
+	dd->auto_eject = dd->prop_auto_eject;
+
 	dd->scale_applet = dd->prop_scale_applet;
 	dd->autofs_friendly = dd->prop_autofs_friendly;
 	new_file = gtk_entry_get_text(GTK_ENTRY(dd->mount_point_entry));
 	if (dd->mount_point) g_free(dd->mount_point);
 	dd->mount_point = g_strdup(new_file);
+	sync_mount_base(dd);
         dd->interval = dd->prop_interval;
 	if (dd->device_pixmap != dd->prop_device_pixmap)
 		{
@@ -172,8 +214,7 @@ static void property_apply_cb( GtkWidget *widget, void *data, DriveData *dd)
 	/*make the panel save our config*/
 	applet_widget_sync_config(APPLET_WIDGET(dd->applet));
         return;
-        widget = NULL;
-	data = NULL;
+        propertybox = NULL;
 }
 
 static gint property_destroy_cb( GtkWidget *w, DriveData *dd)
@@ -217,6 +258,7 @@ void property_show(AppletWidget *applet, gpointer data)
 	dd->prop_device_pixmap = dd->device_pixmap;
 	dd->prop_autofs_friendly = dd->autofs_friendly;
 	dd->prop_scale_applet = dd->scale_applet;
+	dd->prop_auto_eject = dd->auto_eject;
 
 	dd->propwindow = gnome_property_box_new();
 	gtk_window_set_title(GTK_WINDOW(GNOME_PROPERTY_BOX(dd->propwindow)),
@@ -343,6 +385,12 @@ void property_show(AppletWidget *applet, gpointer data)
         gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) scale_applet_cb, dd);
         gtk_widget_show(button);
 
+	button = gtk_check_button_new_with_label (_("Eject on unmount"));
+	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), dd->auto_eject);
+	gtk_signal_connect (GTK_OBJECT(button),"clicked",(GtkSignalFunc) auto_eject_applet_cb, dd);
+	gtk_widget_show(button);
+
 	button = gtk_check_button_new_with_label (_("Use automount friendly status test"));
         gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
         gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), dd->prop_autofs_friendly);
@@ -367,3 +415,4 @@ void property_show(AppletWidget *applet, gpointer data)
 	applet = NULL;
 }
 
+
