@@ -1,4 +1,4 @@
-/* $Id: gweather-applet.c,v 1.21 2000/05/04 12:23:58 jberkman Exp $ */
+/* $Id: gweather-applet.c,v 1.24 2001/01/26 03:45:19 ats Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -177,6 +177,10 @@ void gweather_applet_create (int argc, char *argv[])
                        NULL, 0, NULL);
     gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gweather/tstorm.xpm");
 
+    /* PUSH */
+    gtk_widget_push_visual (gdk_rgb_get_visual ());
+    gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+
     if ((gweather_applet = applet_widget_new("gweather")) == NULL)
         g_error(_("Cannot create applet!\n"));
 
@@ -256,6 +260,10 @@ void gweather_applet_create (int argc, char *argv[])
     place_widgets();
     
     gtk_widget_show(gweather_applet);
+
+    /* POP */
+    gtk_widget_pop_colormap ();
+    gtk_widget_pop_visual ();
 }
 
 
@@ -289,6 +297,7 @@ void gweather_info_load (const gchar *path)
     /* fprintf(stderr, "gweather_info_save: %s\n", prefix); */
     g_free(prefix);
 
+    weather_info_free (gweather_info);
     gweather_info = weather_info_config_read();
 
     gnome_config_pop_prefix();
@@ -306,8 +315,16 @@ static gint timeout_cb (gpointer data)
 static void update_finish (WeatherInfo *info)
 {
     char *s;
-    /* Save weather info */
-    gweather_info = info;
+
+    if (info != gweather_info) {
+	    if (gweather_info != NULL) {
+		    weather_info_free (gweather_info);
+		    gweather_info = NULL;
+	    }
+
+	    /* Save weather info */
+	    gweather_info = weather_info_clone (info);
+    }
 
     /* Store current conditions */
     /* gweather_info_save(APPLET_WIDGET(gweather_applet)->privcfgpath); */
@@ -345,7 +362,7 @@ void gweather_update (void)
     /* Let user know we are updating */
     weather_info_get_pixmap_mini(gweather_info, &cond_pixmap, &cond_mask);
     gtk_pixmap_set(GTK_PIXMAP(pixmap), cond_pixmap, cond_mask);
-    gtk_tooltips_set_tip(tooltips, gweather_applet, "Updating...", NULL);
+    gtk_tooltips_set_tip(tooltips, gweather_applet, _("Updating..."), NULL);
 
     /* Set preferred units */
     weather_units_set(gweather_pref.use_metric ? UNITS_METRIC : UNITS_IMPERIAL);
@@ -361,22 +378,13 @@ void gweather_update (void)
         weather_proxy_set(gweather_pref.proxy_url, gweather_pref.proxy_user, gweather_pref.proxy_passwd);
 
     /* Update current conditions */
-    if (gweather_pref.update_enabled) {
-        if (gweather_info && weather_location_equal(gweather_info->location, gweather_pref.location)) {
-            update_success = weather_info_update(gweather_info, update_finish);
-        } else {
-            weather_info_free(gweather_info);
-            update_success = weather_info_new(gweather_pref.location, update_finish);
-        }
-        if (!update_success)
-            gnome_error_dialog("Update failed! Maybe another already in progress?");  /* FIX */
+    if (gweather_info && weather_location_equal(gweather_info->location, gweather_pref.location)) {
+        update_success = weather_info_update(gweather_info, update_finish);
     } else {
-        if (gweather_info) {
-            if (gweather_pref.use_metric)
-                weather_info_to_metric(gweather_info);
-            else
-		  weather_info_to_imperial(gweather_info);
-	}
-        update_finish(gweather_info);
+        weather_info_free(gweather_info);
+        gweather_info = NULL;
+        update_success = weather_info_new(gweather_pref.location, update_finish);
     }
+    if (!update_success)
+        gnome_error_dialog("Update failed! Maybe another already in progress?");  /* FIX */
 }
