@@ -1,4 +1,4 @@
-/* $Id: gweather-pref.c,v 1.11 2000/04/25 03:02:01 dmueth Exp $ */
+/* $Id: gweather-pref.c,v 1.13 2001/02/11 20:09:58 jleach Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -319,14 +319,13 @@ static void gweather_pref_create (void)
   GtkWidget *apply_button;
   GtkWidget *cancel_button;
   GtkWidget *help_button;
-  GtkWidget *pref_loc_scroll;
-  GtkObject *pref_loc_adj;
+  GtkWidget *scrolled_window;
 
   g_return_if_fail(pref == NULL);
 
-  pref = gnome_dialog_new (_("Gweather Properties"), NULL);
+  pref = gnome_dialog_new (_("GNOME Weather Properties"), NULL);
   gtk_widget_set_usize (pref, -2, 280);
-  gtk_window_set_policy (GTK_WINDOW (pref), FALSE, FALSE, FALSE);
+  gtk_window_set_policy (GTK_WINDOW (pref), TRUE, TRUE, FALSE);
   gnome_dialog_close_hides(GNOME_DIALOG(pref), TRUE);
 
   pref_vbox = GNOME_DIALOG (pref)->vbox;
@@ -497,21 +496,21 @@ static void gweather_pref_create (void)
   gtk_widget_show (pref_loc_hbox);
   gtk_container_add (GTK_CONTAINER (pref_notebook), pref_loc_hbox);
 
-  pref_loc_adj = gtk_adjustment_new(0, 0, 0, 0, 0, 0);
+  scrolled_window = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
+				  GTK_POLICY_AUTOMATIC,
+				  GTK_POLICY_AUTOMATIC);
 
   pref_loc_ctree = gtk_ctree_new (1, 0);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), pref_loc_ctree);
   gtk_widget_show (pref_loc_ctree);
-  gtk_box_pack_start (GTK_BOX (pref_loc_hbox), pref_loc_ctree, TRUE, TRUE, 0);
+  gtk_widget_show (scrolled_window);
+  gtk_box_pack_start (GTK_BOX (pref_loc_hbox), scrolled_window, TRUE, TRUE, 0);
   gtk_clist_set_column_width (GTK_CLIST (pref_loc_ctree), 0, 80);
   gtk_clist_set_selection_mode (GTK_CLIST (pref_loc_ctree), GTK_SELECTION_BROWSE);
   gtk_clist_column_titles_hide (GTK_CLIST (pref_loc_ctree));
-  gtk_clist_set_vadjustment (GTK_CLIST (pref_loc_ctree), GTK_ADJUSTMENT (pref_loc_adj));
   load_locations();
 
-  pref_loc_scroll = gtk_vscrollbar_new (GTK_ADJUSTMENT (pref_loc_adj));
-  gtk_widget_show (pref_loc_scroll);
-  gtk_box_pack_start (GTK_BOX (pref_loc_hbox), pref_loc_scroll, FALSE, FALSE, 0);
-
   pref_loc_note_lbl = gtk_label_new (_("Location"));
   gtk_widget_show (pref_loc_note_lbl);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (pref_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (pref_notebook), 2), pref_loc_note_lbl);
