@@ -1,4 +1,4 @@
-/* $Id: weather.c,v 1.17 2000/06/01 20:57:19 jleach Exp $ */
+/* $Id: weather.c,v 1.22 2001/01/26 03:45:32 ats Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -26,6 +26,8 @@
 #include <unistd.h>
 
 #include <gnome.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gdk-pixbuf/gdk-pixbuf-loader.h>
 #include <ghttp.h>
 
 #include "http.h"
@@ -63,16 +65,15 @@ WeatherLocation *weather_location_new (const gchar *name, const gchar *code, con
 {
     WeatherLocation *location;
 
-    g_return_val_if_fail(strlen(name) <= WEATHER_LOCATION_NAME_MAX_LEN, NULL);
-    g_return_val_if_fail(strlen(code) == WEATHER_LOCATION_CODE_LEN, NULL);
-    g_return_val_if_fail(strlen(zone) == WEATHER_LOCATION_ZONE_LEN, NULL);
-    g_return_val_if_fail(strlen(radar) == WEATHER_LOCATION_RADAR_LEN, NULL);
-
     location = g_new(WeatherLocation, 1);
-    strcpy(location->name, name);
-    strcpy(location->code, code);
-    strcpy(location->zone, zone);
-    strcpy(location->radar, radar);
+    strncpy(location->name, name, WEATHER_LOCATION_NAME_MAX_LEN);
+    location->name[WEATHER_LOCATION_NAME_MAX_LEN] = '\0';
+    strncpy(location->code, code, WEATHER_LOCATION_CODE_LEN);
+    location->code[WEATHER_LOCATION_CODE_LEN] = '\0';
+    strncpy(location->zone, zone, WEATHER_LOCATION_ZONE_LEN);
+    location->zone[WEATHER_LOCATION_ZONE_LEN] = '\0';
+    strncpy(location->radar, radar, WEATHER_LOCATION_RADAR_LEN);
+    location->radar[WEATHER_LOCATION_RADAR_LEN] = '\0';
 
     return location;
 }
@@ -104,7 +105,13 @@ WeatherLocation *weather_location_config_read (gchar *prefix)
     prefix_with_default = g_strconcat(prefix, "=Pittsburgh KPIT PAZ021 pit", NULL);
 
     gnome_config_get_vector(prefix_with_default, &nlocdata, &locdata);
-    g_return_val_if_fail(nlocdata == 4, NULL);
+
+    if (nlocdata != 4) {
+	    g_warning (_("Location vector needs to be 4 elements long"));
+	    g_free(prefix_with_default);
+	    g_strfreev(locdata);
+	    return NULL;
+    }
     location = weather_location_new(locdata[0], locdata[1], locdata[2], locdata[3]);
     g_strfreev(locdata);
 
@@ -133,7 +140,7 @@ gboolean weather_location_equal (const WeatherLocation *location1, const Weather
     return (strcmp(location1->code, location2->code) == 0);
 }
 
-static gchar *wind_direction_str[] = {
+static const gchar *wind_direction_str[] = {
     N_("Variable"),
     N_("North"), N_("North - NorthEast"), N_("Northeast"), N_("East - NorthEast"),
     N_("East"), N_("East - Southeast"), N_("Southeast"), N_("South - Southeast"),
@@ -141,12 +148,16 @@ static gchar *wind_direction_str[] = {
     N_("West"), N_("West - Northwest"), N_("Northwest"), N_("North - Northwest")
 };
 
-gchar *weather_wind_direction_string (WeatherWindDirection wind)
+const gchar *weather_wind_direction_string (WeatherWindDirection wind)
 {
-    return _(wind_direction_str[(int)wind]);
+	if (wind < 0 ||
+	    wind >= (sizeof (wind_direction_str) / sizeof (char *)))
+		return _("Invalid");
+
+	return _(wind_direction_str[(int)wind]);
 }
 
-static gchar *sky_str[] = {
+static const gchar *sky_str[] = {
     N_("Clear Sky"),
     N_("Broken clouds"),
     N_("Scattered clouds"),
@@ -154,9 +165,13 @@ static gchar *sky_str[] = {
     N_("Overcast")
 };
 
-gchar *weather_sky_string (WeatherSky sky)
+const gchar *weather_sky_string (WeatherSky sky)
 {
-    return _(sky_str[(int)sky]);
+	if (sky < 0 ||
+	    sky >= (sizeof (sky_str) / sizeof (char *)))
+		return _("Invalid");
+
+	return _(sky_str[(int)sky]);
 }
 
 
@@ -172,12 +187,16 @@ gchar *weather_sky_string (WeatherSky sky)
  * ever comes up in the weather conditions field, let me know...
  */
 
-static gchar *conditions_str[24][13] = {
+/*
+ * Note, magic numbers, when you change the size here, make sure to change
+ * the below function so that new values are recognized
+ */
+static const gchar *conditions_str[24][13] = {
 /*                   NONE                         VICINITY                             LIGHT                      MODERATE                      HEAVY                      SHALLOW                      PATCHES                         PARTIAL                      THUNDERSTORM                    BLOWING                      SHOWERS                         DRIFTING                      FREEZING                      */
 /*               *******************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************************/
 /* NONE          */ {"",                          "",                                  "",                        "",                           "",                        "",                          "",                             "",                          "",                             "",                          "",                             "",                           "",                          },
 /* DRIZZLE       */ {N_("Drizzle"),               N_("Drizzle in the vicinity"),       N_("Light drizzle"),       N_("Moderate drizzle"),       N_("Heavy drizzle"),       N_("Shallow drizzle"),       N_("Patches of drizzle"),       N_("Partial drizzle"),       N_("Thunderstorm"),             N_("Windy drizzle"),         N_("Showers"),                  N_("Drifting drizzle"),       N_("Freezing drizzle")       },
-/* RAIN          */ {N_("Rain "),                 N_("Rain in the vicinity") ,         N_("Light rain"),          N_("Moderate rain"),          N_("Heavy rain"),          N_("Shallow rain"),          N_("Patches of rain"),          N_("Partial rainfall"),      N_("Thunderstorm"),             N_("Blowing rainfall"),      N_("Rain showers"),             N_("Drifting rain"),          N_("Freezing rain")          },
+/* RAIN          */ {N_("Rain"),                  N_("Rain in the vicinity") ,         N_("Light rain"),          N_("Moderate rain"),          N_("Heavy rain"),          N_("Shallow rain"),          N_("Patches of rain"),          N_("Partial rainfall"),      N_("Thunderstorm"),             N_("Blowing rainfall"),      N_("Rain showers"),             N_("Drifting rain"),          N_("Freezing rain")          },
 /* SNOW          */ {N_("Snow"),                  N_("Snow in the vicinity") ,         N_("Light snow"),          N_("Moderate snow"),          N_("Heavy snow"),          N_("Shallow snow"),          N_("Patches of snow"),          N_("Partial snowfall"),      N_("Snowstorm"),                N_("Blowing snowfall"),      N_("Snow showers"),             N_("Drifting snow"),          N_("Freezing snow")          },
 /* SNOW_GRAINS   */ {N_("Snow grains"),           N_("Snow grains in the vicinity") ,  N_("Light snow grains"),   N_("Moderate snow grains"),   N_("Heavy snow grains"),   N_("Shallow snow grains"),   N_("Patches of snow grains"),   N_("Partial snow grains"),   N_("Snowstorm"),                N_("Blowing snow grains"),   N_("Snow grain showers"),       N_("Drifting snow grains"),   N_("Freezing snow grains")   },
 /* ICE_CRYSTALS  */ {N_("Ice crystals"),          N_("Ice crystals in the vicinity") , N_("Few ice crystals"),    N_("Moderate ice crystals"),  N_("Heavy ice crystals"),  "??",                        N_("Patches of ice crystals"),  N_("Partial ice crystals"),  N_("Ice crystal storm"),        N_("Blowing ice crystals"),  N_("Showers of ice crystals"),  N_("Drifting ice crystals"),  N_("Freezing ice crystals")  },
@@ -201,15 +220,21 @@ static gchar *conditions_str[24][13] = {
 /* DUST_WHIRLS   */ {N_("Dust whirls"),           N_("Dust whirls in the vicinity") ,  N_("Light dust whirls"),   N_("Moderate dust whirls"),   N_("Heavy dust whirls"),   N_("Shallow dust whirls"),   N_("Patches of dust whirls"),   N_("Partial dust whirls"),   "??",                           N_("Blowing dust whirls"),   "??",                           N_("Drifting dust whirls"),   "??"                         }
 };
 
-gchar *weather_conditions_string (WeatherConditions cond)
+const gchar *weather_conditions_string (WeatherConditions cond)
 {
-    gchar *str;
+    const gchar *str;
 
     if (!cond.significant) {
-        return "-";
+	    return "-";
     } else {
-        str = _(conditions_str[(int)cond.phenomenon][(int)cond.qualifier]);
-        return (strlen(str) > 0) ? str : "-";
+	    if (cond.phenomenon >= 0 &&
+		cond.phenomenon < 24 &&
+		cond.qualifier >= 0 &&
+		cond.qualifier < 13)
+		    str = _(conditions_str[(int)cond.phenomenon][(int)cond.qualifier]);
+	    else
+		    str = _("Invalid");
+	    return (strlen(str) > 0) ? str : "-";
     }
 }
 
@@ -219,7 +244,9 @@ static WeatherInfoFunc request_cb = NULL;
 static WeatherInfo *request_info = NULL;
 static ghttp_request *metar_request = NULL,
                      *iwin_request = NULL,
-                     *wx_request = NULL;
+                     *wx_request = NULL,
+                     *met_request = NULL;
+                     
 static gboolean requests_pending = FALSE;
 
 static __inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
@@ -227,7 +254,7 @@ static __inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
     if (requests_pending)
         return FALSE;
 
-    g_assert(!metar_request && !iwin_request && !wx_request);
+    g_assert(!metar_request && !iwin_request && !wx_request && !met_request);
     g_assert(!request_info && !request_cb);
 
     requests_pending = TRUE;
@@ -237,7 +264,7 @@ static __inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
     return TRUE;
 }
 
-static __inline void request_done (ghttp_request **req)
+static inline void request_done (ghttp_request **req)
 {
     g_return_if_fail(req != NULL);
 
@@ -245,11 +272,11 @@ static __inline void request_done (ghttp_request **req)
     *req = NULL;
 }
 
-static __inline void requests_done_check (void)
+static inline void requests_done_check (void)
 {
     g_return_if_fail(requests_pending);
 
-    if (!metar_request && !iwin_request && !wx_request) {
+    if (!metar_request && !iwin_request && !wx_request && !met_request) {
         requests_pending = FALSE;
         /* Next two lines temporarily here */
         if (weather_units == UNITS_METRIC)
@@ -296,7 +323,7 @@ static void metar_init_re (void)
     regcomp(&metar_re[COND_RE], COND_RE_STR, REG_EXTENDED);
 }
 
-static __inline gint days_in_month (gint month, gint year)
+static inline gint days_in_month (gint month, gint year)
 {
     if (month == 1)
         return ((year % 4) == 0) ? 29 : 28;
@@ -551,7 +578,7 @@ static gboolean metar_tok_pres (gchar *tokp, WeatherInfo *info)
 /* Relative humidity computation - thanks to <Olof.Oberg@modopaper.modogroup.com> */
 
 
-static __inline gint calc_humidity(gdouble temp, gdouble dewp)
+static inline gint calc_humidity(gdouble temp, gdouble dewp)
 {
     gdouble esat, esurf;
 
@@ -607,7 +634,8 @@ static gboolean metar_tok_cond (gchar *tokp, WeatherInfo *info)
     squal[pphen - tokp] = 0;
 
     memset(sphen, 0, sizeof(sphen));
-    strcpy(sphen, pphen);
+    strncpy(sphen, pphen, sizeof(sphen));
+    sphen[sizeof(sphen)-1] = '\0';
 
     /* Defaults */
     info->cond.qualifier = QUALIFIER_NONE;
@@ -766,34 +794,44 @@ static void metar_get_finish (ghttp_request *req, ghttp_status status, gpointer
     g_return_if_fail(info != NULL);
     info->forecast = NULL;
     loc = info->location;
-    g_return_if_fail(loc != NULL);
+    if (loc == NULL) {
+	    g_warning (_("WeatherInfo missing location"));
+	    request_done(&metar_request);
+	    requests_done_check();
+	    return;
+    }
 
     if (status == ghttp_error) {
         g_warning(_("Failed to get METAR data.\n"));
     } else {
         body = ghttp_get_body(metar_request);
         body_len = ghttp_get_body_len(metar_request);
-        g_return_if_fail(body != NULL);
-        g_return_if_fail(body_len > 0);
+
+	if (body == NULL ||
+	    body_len <= 0) {
+		request_done(&metar_request);
+		requests_done_check();
+		return;
+	}
+
         body[body_len-1] = 0;  /* quick hack */
 
-        sprintf(searchkey, "\n%s", loc->code);
+        g_snprintf (searchkey, sizeof (searchkey), "\n%s", loc->code);
         metar = strstr(body, searchkey);
         if (metar == NULL) {
             success = FALSE;
         } else {
             metar += WEATHER_LOCATION_CODE_LEN + 2;
             eoln = strchr(metar, '\n');
-            g_return_if_fail(eoln !=  NULL);
-
-            *eoln = 0;
+	    if (eoln != NULL)
+		    *eoln = 0;
             success = metar_parse(metar, info);
-            *eoln = '\n';
+	    if (eoln != NULL)
+		    *eoln = '\n';
         }
 
         info->valid = success;
     }
-
     request_done(&metar_request);
     requests_done_check();
 }
@@ -807,7 +845,10 @@ static void metar_get_start (WeatherInfo *info)
     g_return_if_fail(info != NULL);
     info->valid = FALSE;
     loc = info->location;
-    g_return_if_fail(loc != NULL);
+    if (loc == NULL) {
+	    g_warning (_("WeatherInfo missing location"));
+	    return;
+    }
 
     metar_request = ghttp_request_new();
     url = g_strdup_printf("http://weather.noaa.gov/cgi-bin/mgetmetar.pl?cccc=%s", loc->code);
@@ -908,13 +949,21 @@ static gchar *iwin_parse (gchar *iwin, WeatherLocation *loc)
 
     g_return_val_if_fail(iwin != NULL, NULL);
     g_return_val_if_fail(loc != NULL, NULL);
-    g_return_val_if_fail(loc->name[0] != '-', NULL);
+    if (loc->name[0] == '-')
+        return NULL;
 
     /* fprintf(stderr, "iwin_parse\n"); */
 
     iwin_init_re();
 
     p = iwin;
+
+    /* Strip CR from CRLF input */
+    while ((p = strchr(p, '\r')) != NULL)
+        *p = ' ';
+
+    p = iwin;
+    
     while ((ret = regexec(&iwin_re, p, 1, match, 0)) != REG_NOMATCH) {
         rangep = p + match[0].rm_so;
         p = strchr(rangep, '\n');
@@ -946,17 +995,28 @@ static void iwin_get_finish (ghttp_request *req, ghttp_status status, gpointer d
     g_return_if_fail(req == iwin_request);
 
     g_return_if_fail(info != NULL);
+    g_free (info->forecast);
     info->forecast = NULL;
     loc = info->location;
-    g_return_if_fail(loc != NULL);
+    if (loc == NULL) {
+	    g_warning (_("WeatherInfo missing location"));
+	    request_done(&iwin_request);
+	    requests_done_check();
+	    return;
+    }
 
     if (status == ghttp_error) {
         g_warning(_("Failed to get IWIN forecast data.\n"));
     } else {
         body = ghttp_get_body(iwin_request);
         body_len = ghttp_get_body_len(iwin_request);
-        g_return_if_fail(body != NULL);
-        g_return_if_fail(body_len > 0);
+
+	if (body == NULL ||
+	    body_len <= 0) {
+		request_done(&iwin_request);
+		requests_done_check();
+		return;
+	}
         body[body_len-1] = 0;  /* quick hack */
 
         forecast = iwin_parse(body, loc);
@@ -967,6 +1027,218 @@ static void iwin_get_finish (ghttp_request *req, ghttp_status status, gpointer d
     requests_done_check();
 }
 
+static char *met_reprocess(char *x, int len)
+{
+	char *p = x;
+	char *o;
+	int spacing = 0;
+	static gchar *buf;
+	static gint buflen = 0;
+	gchar *lastspace = NULL;
+	int count = 0;
+	int lastcount = 0;
+	
+	if(buflen < len)
+	{
+		if(buf)
+			g_free(buf);
+		buf=g_malloc(len);
+		buflen=len;
+	}
+	memcpy(buf, x, len);
+		
+	o=buf;
+
+	while(*p)
+	{
+		if(isspace(*p))
+		{
+			spacing=1;
+			p++;
+			continue;
+		}
+		if(spacing)
+		{
+			if(count>75)
+			{
+				if(lastspace)
+					*lastspace = '\n';
+				count -= lastcount;
+			}
+			lastspace = o;
+			lastcount = count;
+			*o++=' ';
+			spacing=0;
+		}
+
+		if(*p=='&')
+		{
+			if(strncasecmp(p, "&amp;", 5)==0)
+			{
+				*o++='&';
+				count++;
+				p+=5;
+				continue;
+			}
+			if(strncasecmp(p, "&lt;", 4)==0)
+			{
+				*o++='<';
+				count++;
+				p+=4;
+				continue;
+			}
+			if(strncasecmp(p, "&gt;", 4)==0)
+			{
+				*o++='>';
+				count++;
+				p+=4;
+				continue;
+			}
+		}
+		if(*p=='<')
+		{
+			if(strncasecmp(p, "<BR>", 4)==0)
+			{
+				*o++='\n';
+				count=0;
+				lastspace = NULL;
+				p+=4;
+				continue;
+			}
+			break;
+		}
+		*o++=*p++;
+		count++;
+	}
+	*o=0;
+	return buf;
+}
+
+
+/*
+ * Parse the metoffice forecast info.
+ * For gnome 2.0 we want to just embed an HTML bonobo component and 
+ * be done with this ;) 
+ */
+
+static gchar *met_parse (gchar *meto, WeatherLocation *loc)
+{ 
+    gchar *p;
+    gchar *rp;
+    gchar *r = g_strdup("Met Office Forecast\n");
+    gchar *t;    
+    gint i=0;
+
+    static char *key[]=
+    {
+    	"<!-- <!TODAY_START> -->",
+    	"<!-- <!TONIGHT_START> -->",
+    	"<!-- <!TOMORROW_START> -->",
+    	"<!-- <!OUTLOOK_START> -->",
+    	NULL
+    };
+    static char *keyend[]=
+    {
+    	"<!-- <!TODAY_END> -->",
+    	"<!-- <!TONIGHT_END> -->",
+    	"<!-- <!TOMORROW_END> -->",
+    	"<!-- <!OUTLOOK_END> -->",
+    	NULL
+    };
+    static char *name[4]=
+    {
+    	"Today:",
+    	"Tonight:",
+    	"Tomorrow:",
+    	"Outlook:"
+    };
+    
+    
+    while(key[i]!=NULL)
+    {
+    	p=strstr(meto, key[i]);
+    	if(p==NULL)
+    	{
+    		printf("No %s\n", key[i]);
+    		i++;
+    		continue;
+    	}
+    	p+=strlen(key[i]);
+
+    	rp = strstr(p, keyend[i]);
+	if(rp==NULL)
+	{
+    		printf("No %s\n", keyend[i]);
+		i++;
+		continue;
+	}
+	
+	/* p to rp is the text block we want but in HTML malformat */    	
+
+    	t = g_strconcat(r, name[i], "\n", met_reprocess(p, rp-p), "\n", NULL);
+    	
+    	g_free(r);
+    	r = t;
+    	i++;
+    }
+    return r;
+}
+
+static void met_get_finish (ghttp_request *req, ghttp_status status, gpointer data)
+{
+    WeatherInfo *info = (WeatherInfo *)data;
+    WeatherLocation *loc;
+    gchar *body;
+    gint body_len;
+    gchar *forecast;
+
+    g_return_if_fail(req == met_request);
+
+    g_return_if_fail(info != NULL);
+    info->forecast = NULL;
+    loc = info->location;
+    g_return_if_fail(loc != NULL);
+
+    if (status == ghttp_error) {
+        g_warning(_("Failed to get Met Office forecast data.\n"));
+    } else {
+        body = ghttp_get_body(met_request);
+        body_len = ghttp_get_body_len(met_request);
+        if(body==NULL || body_len == 0)
+        {
+	    request_done(&iwin_request);
+	    requests_done_check();
+	    return;
+	}
+        
+        body[body_len-1] = 0;  /* quick hack */
+
+        forecast = met_parse(body, loc);
+        info->forecast = forecast;
+    }
+
+    request_done(&met_request);
+    requests_done_check();
+}
+
+static void metoffice_get_start (WeatherInfo *info)
+{
+    gchar *url;
+    WeatherLocation *loc;
+    loc = info->location;
+
+    met_request = ghttp_request_new();
+    
+    url = g_strdup_printf("http://www.metoffice.gov.uk/datafiles/%s.html", loc->zone+1);
+    ghttp_set_uri(met_request, url);
+    g_free(url);
+    ghttp_set_proxy(met_request, weather_proxy_url);
+    ghttp_set_proxy_authinfo(met_request, weather_proxy_user, weather_proxy_passwd);
+    ghttp_set_header(met_request, http_hdr_Connection, "close");
+
+    http_process_bg(met_request, met_get_finish, info);
+}	
+
 /* Get forecast into newly alloc'ed string */
 static void iwin_get_start (WeatherInfo *info)
 {
@@ -981,6 +1253,12 @@ static void iwin_get_start (WeatherInfo *info)
 
     if (loc->zone[0] == '-')
         return;
+        
+    if (loc->zone[0] == ':')	/* Met Office Region Names */
+    {
+    	metoffice_get_start (info);
+    	return;
+    }
 
     strncpy(state, loc->zone, 2);
     state[2] = 0;
@@ -1001,21 +1279,25 @@ static void iwin_get_start (WeatherInfo *info)
 static GdkPixmap *wx_construct (gpointer data, gint data_len)
 {
     GdkPixmap *pixmap;
-    char *tmpname = g_strdup(tmpnam(NULL));
-    FILE *fp;
+    GdkPixbuf *pixbuf;
+    GdkPixbufLoader *pixbuf_loader;
 
     /* fprintf(stderr, "wx_get\n"); */
-    /* fprintf(stderr, "tmpname: %s\n", tmpname); */
 
-    fp = fopen(tmpname, "wb");
-    g_return_val_if_fail(fp != NULL, NULL);
-    fwrite(data, data_len, 1, fp);
-    fclose(fp);
+    pixbuf_loader = gdk_pixbuf_loader_new ();
 
-    gdk_imlib_load_file_to_pixmap(tmpname, &pixmap, NULL);
+    gdk_pixbuf_loader_write (pixbuf_loader, data, data_len);
+    gdk_pixbuf_loader_close (pixbuf_loader);
 
-    unlink(tmpname);
-    g_free(tmpname);
+    pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
+
+    pixmap = NULL;
+
+    if (pixbuf != NULL) {
+	    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, NULL, 127);
+    }
+
+    gtk_object_unref (GTK_OBJECT (pixbuf_loader));
 
     return pixmap;
 }
@@ -1087,7 +1369,11 @@ gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, Weath
     } else {
         location = info->location;
         g_free(info->forecast);
-        gdk_imlib_free_pixmap(info->radar);
+        info->forecast = NULL;
+	if (info->radar != NULL) {
+		gdk_pixmap_unref (info->radar);
+		info->radar = NULL;
+	}
     }
 
     /* FIX */
@@ -1144,6 +1430,8 @@ void weather_info_config_write (WeatherInfo *info)
     gnome_config_set_float("visibility", info->visibility);
     if (info->forecast)
         gnome_config_set_string("forecast", info->forecast);
+    else
+        gnome_config_set_string("forecast", "");
     /* info->radar = NULL; */
 }
 
@@ -1177,11 +1465,23 @@ WeatherInfo *weather_info_clone (const WeatherInfo *info)
     WeatherInfo *clone;
     
     g_return_val_if_fail(info != NULL, NULL);
+
     clone = g_new(WeatherInfo, 1);
+
+
+    /* move everything */
     g_memmove(clone, info, sizeof(WeatherInfo));
+
+
+    /* special moves */
     clone->location = weather_location_clone(info->location);
-    if (info->forecast)
-        clone->forecast = g_strdup(info->forecast);
+    /* This handles null correctly */
+    clone->forecast = g_strdup(info->forecast);
+
+    clone->radar = info->radar;
+    if (clone->radar != NULL)
+	    gdk_pixmap_ref (clone->radar);
+
     return clone;
 }
 
@@ -1189,8 +1489,13 @@ void weather_info_free (WeatherInfo *info)
 {
     if (info) {
         g_free(info->location);
+        info->location = NULL;
         g_free(info->forecast);
-        gdk_imlib_free_pixmap(info->radar);
+        info->forecast = NULL;
+	if (info->radar != NULL) {
+		gdk_pixmap_unref (info->radar);
+		info->radar = NULL;
+	}
     }
     g_free(info);
 }
@@ -1234,11 +1539,11 @@ void weather_proxy_set (const gchar *url, const gchar *user, const gchar *passwd
     g_free(weather_proxy_passwd); weather_proxy_passwd = NULL;
 
     if (url && (strlen(url) > 0))
-        weather_proxy_url = strdup(url);
+        weather_proxy_url = g_strdup(url);
     if (user && (strlen(user) > 0))
-        weather_proxy_user = strdup(user);
+        weather_proxy_user = g_strdup(user);
     if (passwd && (strlen(passwd) > 0))
-        weather_proxy_passwd = strdup(passwd);
+        weather_proxy_passwd = g_strdup(passwd);
 }
 
 void weather_info_to_metric (WeatherInfo *info)
@@ -1278,14 +1583,14 @@ void weather_info_to_imperial (WeatherInfo *info)
 }
 
 
-gchar *weather_info_get_location (WeatherInfo *info)
+const gchar *weather_info_get_location (WeatherInfo *info)
 {
     g_return_val_if_fail(info != NULL, NULL);
     g_return_val_if_fail(info->location != NULL, NULL);
     return info->location->name;
 }
 
-gchar *weather_info_get_update (WeatherInfo *info)
+const gchar *weather_info_get_update (WeatherInfo *info)
 {
     static gchar buf[200];
 
@@ -1297,15 +1602,18 @@ gchar *weather_info_get_update (WeatherInfo *info)
     if (info->update != 0) {
         struct tm *tm;
         tm = localtime(&info->update);
-        strftime(buf, 90, _("%a, %b %d / %H:%M"), tm);
+        if (strftime(buf, sizeof(buf), _("%a, %b %d / %H:%M"), tm) <= 0)
+		strcpy (buf, "???");
+	buf[sizeof(buf)-1] = '\0';
     } else {
-        strcpy(buf, _("Unknown observation time"));
+        strncpy(buf, _("Unknown observation time"), sizeof (buf));
+	buf[sizeof(buf)-1] = '\0';
     }
 
     return buf;
 }
 
-gchar *weather_info_get_sky (WeatherInfo *info)
+const gchar *weather_info_get_sky (WeatherInfo *info)
 {
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
@@ -1313,7 +1621,7 @@ gchar *weather_info_get_sky (WeatherInfo *info)
     return weather_sky_string(info->sky);
 }
 
-gchar *weather_info_get_conditions (WeatherInfo *info)
+const gchar *weather_info_get_conditions (WeatherInfo *info)
 {
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
@@ -1321,61 +1629,63 @@ gchar *weather_info_get_conditions (WeatherInfo *info)
     return weather_conditions_string(info->cond);
 }
 
-gchar *weather_info_get_temp (WeatherInfo *info)
+const gchar *weather_info_get_temp (WeatherInfo *info)
 {
     static gchar buf[10];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
-    sprintf(buf, "%.1f%s", info->temp, TEMP_UNIT_STR(info->units));
+    g_snprintf(buf, sizeof (buf), "%.1f%s", info->temp, TEMP_UNIT_STR(info->units));
     return buf;
 }
 
-gchar *weather_info_get_dew (WeatherInfo *info)
+const gchar *weather_info_get_dew (WeatherInfo *info)
 {
     static gchar buf[10];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
-    sprintf(buf, "%.1f%s", info->dew, TEMP_UNIT_STR(info->units));
+    g_snprintf(buf, sizeof (buf), "%.1f%s", info->dew, TEMP_UNIT_STR(info->units));
     return buf;
 }
 
-gchar *weather_info_get_humidity (WeatherInfo *info)
+const gchar *weather_info_get_humidity (WeatherInfo *info)
 {
     static gchar buf[10];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
-    sprintf(buf, "%d%%", info->humidity);
+    g_snprintf(buf, sizeof (buf), "%d%%", info->humidity);
     return buf;
 }
 
-gchar *weather_info_get_wind (WeatherInfo *info)
+const gchar *weather_info_get_wind (WeatherInfo *info)
 {
     static gchar buf[100];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
-    if (info->windspeed == 0.00)
-        strcpy(buf, _("Calm"));
-    else
-        sprintf(buf, "%s / %d %s", weather_wind_direction_string(info->wind),
-                                   info->windspeed, WINDSPEED_UNIT_STR(info->units));
+    if (info->windspeed == 0.00) {
+        strncpy(buf, _("Calm"), sizeof(buf));
+	buf[sizeof(buf)-1] = '\0';
+    } else
+        g_snprintf(buf, sizeof(buf), "%s / %d %s",
+		   weather_wind_direction_string(info->wind),
+		   info->windspeed, WINDSPEED_UNIT_STR(info->units));
     return buf;
 }
 
-gchar *weather_info_get_pressure (WeatherInfo *info)
+const gchar *weather_info_get_pressure (WeatherInfo *info)
 {
     static gchar buf[30];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "-";
-    sprintf(buf, "%.2f %s", info->pressure, PRESSURE_UNIT_STR(info->units));
+    g_snprintf(buf, sizeof (buf), "%.2f %s", info->pressure, PRESSURE_UNIT_STR(info->units));
     return buf;
 }
 
-gchar *weather_info_get_visibility (WeatherInfo *info)
+const gchar *weather_info_get_visibility (WeatherInfo *info)
 {
     static gchar buf[40];
     g_return_val_if_fail(info != NULL, NULL);
@@ -1383,11 +1693,11 @@ gchar *weather_info_get_visibility (WeatherInfo *info)
         return "-";
     if (info->visibility < 0.0)
         return _("Unknown");
-    sprintf(buf, "%.1f %s", info->visibility, VISIBILITY_UNIT_STR(info->units));
+    g_snprintf(buf, sizeof (buf), "%.1f %s", info->visibility, VISIBILITY_UNIT_STR(info->units));
     return buf;
 }
 
-gchar *weather_info_get_forecast (WeatherInfo *info)
+const gchar *weather_info_get_forecast (WeatherInfo *info)
 {
     g_return_val_if_fail(info != NULL, NULL);
     return info->forecast;
@@ -1399,19 +1709,19 @@ GdkPixmap *weather_info_get_radar (WeatherInfo *info)
     return info->radar;
 }
 
-gchar *weather_info_get_temp_summary (WeatherInfo *info)
+const gchar *weather_info_get_temp_summary (WeatherInfo *info)
 {
     static gchar buf[10];
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
         return "--";
-    sprintf(buf, "%d\260", (int)info->temp);
+    g_snprintf(buf, sizeof (buf), "%d\260", (int)info->temp);
     return buf;
 }
 
 gchar *weather_info_get_weather_summary (WeatherInfo *info)
 {
-    gchar *buf;
+    const gchar *buf;
     g_return_val_if_fail(info != NULL, NULL);
     if (!info->valid)
       return g_strdup (_("Retrieval failed"));
@@ -1422,10 +1732,10 @@ gchar *weather_info_get_weather_summary (WeatherInfo *info)
 }
 
 
-static GdkPixmap **weather_pixmaps_mini;
-static GdkBitmap **weather_pixmaps_mini_mask;
-static GdkPixmap **weather_pixmaps;
-static GdkBitmap **weather_pixmaps_mask;
+static GdkPixmap **weather_pixmaps_mini = NULL;
+static GdkBitmap **weather_pixmaps_mini_mask = NULL;
+static GdkPixmap **weather_pixmaps = NULL;
+static GdkBitmap **weather_pixmaps_mask = NULL;
 
 #include "pixmaps/unknown-mini.xpm"
 #include "pixmaps/sun-mini.xpm"
@@ -1456,6 +1766,21 @@ static GdkBitmap **weather_pixmaps_mask;
 
 #define NUM_PIX       8
 
+static void
+xpm_to_pixmap (char **xpm_data, GdkPixmap **pixmap, GdkBitmap **mask)
+{
+	GdkPixbuf *pixbuf = gdk_pixbuf_new_from_xpm_data ((const char **)xpm_data);
+
+	*pixmap = NULL;
+	*mask = NULL;
+
+	if (pixbuf != NULL) {
+		gdk_pixbuf_render_pixmap_and_mask (pixbuf, pixmap, mask, 127);
+
+		gdk_pixbuf_unref (pixbuf);
+	}
+}
+
 static void init_pixmaps (void)
 {
     static gboolean initialized = FALSE;
@@ -1466,25 +1791,25 @@ static void init_pixmaps (void)
 
     weather_pixmaps_mini = g_new(GdkPixmap *, NUM_PIX);
     weather_pixmaps_mini_mask = g_new(GdkBitmap *, NUM_PIX);
-    gdk_imlib_data_to_pixmap(unknown_mini_xpm, &weather_pixmaps_mini[PIX_UNKNOWN], &weather_pixmaps_mini_mask[PIX_UNKNOWN]);
-    gdk_imlib_data_to_pixmap(sun_mini_xpm, &weather_pixmaps_mini[PIX_SUN], &weather_pixmaps_mini_mask[PIX_SUN]);
-    gdk_imlib_data_to_pixmap(suncloud_mini_xpm, &weather_pixmaps_mini[PIX_SUNCLOUD], &weather_pixmaps_mini_mask[PIX_SUNCLOUD]);
-    gdk_imlib_data_to_pixmap(cloud_mini_xpm, &weather_pixmaps_mini[PIX_CLOUD], &weather_pixmaps_mini_mask[PIX_CLOUD]);
-    gdk_imlib_data_to_pixmap(rain_mini_xpm, &weather_pixmaps_mini[PIX_RAIN], &weather_pixmaps_mini_mask[PIX_RAIN]);
-    gdk_imlib_data_to_pixmap(tstorm_mini_xpm, &weather_pixmaps_mini[PIX_TSTORM], &weather_pixmaps_mini_mask[PIX_TSTORM]);
-    gdk_imlib_data_to_pixmap(snow_mini_xpm, &weather_pixmaps_mini[PIX_SNOW], &weather_pixmaps_mini_mask[PIX_SNOW]);
-    gdk_imlib_data_to_pixmap(fog_mini_xpm, &weather_pixmaps_mini[PIX_FOG], &weather_pixmaps_mini_mask[PIX_FOG]);
+    xpm_to_pixmap (unknown_mini_xpm, &weather_pixmaps_mini[PIX_UNKNOWN], &weather_pixmaps_mini_mask[PIX_UNKNOWN]);
+    xpm_to_pixmap (sun_mini_xpm, &weather_pixmaps_mini[PIX_SUN], &weather_pixmaps_mini_mask[PIX_SUN]);
+    xpm_to_pixmap (suncloud_mini_xpm, &weather_pixmaps_mini[PIX_SUNCLOUD], &weather_pixmaps_mini_mask[PIX_SUNCLOUD]);
+    xpm_to_pixmap (cloud_mini_xpm, &weather_pixmaps_mini[PIX_CLOUD], &weather_pixmaps_mini_mask[PIX_CLOUD]);
+    xpm_to_pixmap (rain_mini_xpm, &weather_pixmaps_mini[PIX_RAIN], &weather_pixmaps_mini_mask[PIX_RAIN]);
+    xpm_to_pixmap (tstorm_mini_xpm, &weather_pixmaps_mini[PIX_TSTORM], &weather_pixmaps_mini_mask[PIX_TSTORM]);
+    xpm_to_pixmap (snow_mini_xpm, &weather_pixmaps_mini[PIX_SNOW], &weather_pixmaps_mini_mask[PIX_SNOW]);
+    xpm_to_pixmap (fog_mini_xpm, &weather_pixmaps_mini[PIX_FOG], &weather_pixmaps_mini_mask[PIX_FOG]);
 
     weather_pixmaps = g_new(GdkPixmap *, NUM_PIX);
     weather_pixmaps_mask = g_new(GdkBitmap *, NUM_PIX);
-    gdk_imlib_data_to_pixmap(unknown_xpm, &weather_pixmaps[PIX_UNKNOWN], &weather_pixmaps_mask[PIX_UNKNOWN]);
-    gdk_imlib_data_to_pixmap(sun_xpm, &weather_pixmaps[PIX_SUN], &weather_pixmaps_mask[PIX_SUN]);
-    gdk_imlib_data_to_pixmap(suncloud_xpm, &weather_pixmaps[PIX_SUNCLOUD], &weather_pixmaps_mask[PIX_SUNCLOUD]);
-    gdk_imlib_data_to_pixmap(cloud_xpm, &weather_pixmaps[PIX_CLOUD], &weather_pixmaps_mask[PIX_CLOUD]);
-    gdk_imlib_data_to_pixmap(rain_xpm, &weather_pixmaps[PIX_RAIN], &weather_pixmaps_mask[PIX_RAIN]);
-    gdk_imlib_data_to_pixmap(tstorm_xpm, &weather_pixmaps[PIX_TSTORM], &weather_pixmaps_mask[PIX_TSTORM]);
-    gdk_imlib_data_to_pixmap(snow_xpm, &weather_pixmaps[PIX_SNOW], &weather_pixmaps_mask[PIX_SNOW]);
-    gdk_imlib_data_to_pixmap(fog_xpm, &weather_pixmaps[PIX_FOG], &weather_pixmaps_mask[PIX_FOG]);
+    xpm_to_pixmap (unknown_xpm, &weather_pixmaps[PIX_UNKNOWN], &weather_pixmaps_mask[PIX_UNKNOWN]);
+    xpm_to_pixmap (sun_xpm, &weather_pixmaps[PIX_SUN], &weather_pixmaps_mask[PIX_SUN]);
+    xpm_to_pixmap (suncloud_xpm, &weather_pixmaps[PIX_SUNCLOUD], &weather_pixmaps_mask[PIX_SUNCLOUD]);
+    xpm_to_pixmap (cloud_xpm, &weather_pixmaps[PIX_CLOUD], &weather_pixmaps_mask[PIX_CLOUD]);
+    xpm_to_pixmap (rain_xpm, &weather_pixmaps[PIX_RAIN], &weather_pixmaps_mask[PIX_RAIN]);
+    xpm_to_pixmap (tstorm_xpm, &weather_pixmaps[PIX_TSTORM], &weather_pixmaps_mask[PIX_TSTORM]);
+    xpm_to_pixmap (snow_xpm, &weather_pixmaps[PIX_SNOW], &weather_pixmaps_mask[PIX_SNOW]);
+    xpm_to_pixmap (fog_xpm, &weather_pixmaps[PIX_FOG], &weather_pixmaps_mask[PIX_FOG]);
 }
 
 void _weather_info_get_pixmap (WeatherInfo *info, gboolean mini, GdkPixmap **pixmap, GdkBitmap **mask)
