@@ -399,11 +399,12 @@ main(int argc, char **argv)
 					  properties_box,
 					  NULL);
     
-    applet_widget_register_callback(APPLET_WIDGET(applet),
-				    "help",
-				    _("Help"),
-				    show_help,
-				    NULL);
+    applet_widget_register_stock_callback(APPLET_WIDGET(applet),
+					  "help",
+					  GNOME_STOCK_PIXMAP_HELP,
+					  _("Help"),
+					  show_help,
+					  NULL);
 
     applet_widget_register_stock_callback(APPLET_WIDGET(applet),
 					  "about",
