@@ -47,6 +47,9 @@
 #elif HAVE_SYS_SOUNDCARD_H
 #include <sys/soundcard.h>
 #define OSS_API
+#elif HAVE_SOUNDCARD_H
+#include <soundcard.h>
+#define OSS_API
 #elif HAVE_SYS_AUDIOIO_H
 #include <sys/audioio.h>
 #define SUN_API
@@ -732,13 +735,6 @@ main(int argc, char **argv)
 	if (!applet)
 		g_error(("Can't create applet!\n"));
 
-	mixerw = create_mixer_widget();
-	gtk_widget_show(mixerw);
-	applet_widget_add(APPLET_WIDGET(applet), mixerw);
-
-	gtk_object_set_user_data(GTK_OBJECT(applet),
-				gtk_object_get_user_data(GTK_OBJECT(mixerw)));
-
 	gtk_signal_connect(GTK_OBJECT(applet),"change_orient",
 			   GTK_SIGNAL_FUNC(applet_change_orient),
 			   NULL);
@@ -746,6 +742,13 @@ main(int argc, char **argv)
 			   GTK_SIGNAL_FUNC(applet_change_pixel_size),
 			   NULL);
 
+	mixerw = create_mixer_widget();
+	gtk_widget_show(mixerw);
+	applet_widget_add(APPLET_WIDGET(applet), mixerw);
+
+	gtk_object_set_user_data(GTK_OBJECT(applet),
+				gtk_object_get_user_data(GTK_OBJECT(mixerw)));
+
         applet_widget_register_stock_callback (APPLET_WIDGET(applet),
 					       "run_gmix",
 					       GNOME_STOCK_MENU_VOLUME,
